# Generated by roxygen2: do not edit by hand

S3method(bake,step_blur)
S3method(bake,step_pd_degree)
S3method(bake,step_pd_point_cloud)
S3method(bake,step_pd_raster)
S3method(bake,step_vpd_algebraic_functions)
S3method(bake,step_vpd_betti_curve)
S3method(bake,step_vpd_complex_polynomial)
S3method(bake,step_vpd_descriptive_statistics)
S3method(bake,step_vpd_euler_characteristic_curve)
S3method(bake,step_vpd_normalized_life_curve)
S3method(bake,step_vpd_persistence_block)
S3method(bake,step_vpd_persistence_image)
S3method(bake,step_vpd_persistence_landscape)
S3method(bake,step_vpd_persistence_silhouette)
S3method(bake,step_vpd_persistent_entropy_summary)
S3method(bake,step_vpd_tent_template_functions)
S3method(bake,step_vpd_tropical_coordinates)
S3method(birth_range,PHom)
S3method(birth_range,data.frame)
S3method(birth_range,default)
S3method(birth_range,diagram)
S3method(birth_range,matrix)
S3method(birth_range,persistence)
S3method(life_support,PHom)
S3method(life_support,data.frame)
S3method(life_support,default)
S3method(life_support,diagram)
S3method(life_support,matrix)
S3method(life_support,persistence)
S3method(pairs_max,PHom)
S3method(pairs_max,data.frame)
S3method(pairs_max,default)
S3method(pairs_max,diagram)
S3method(pairs_max,matrix)
S3method(pairs_max,persistence)
S3method(pairs_min,PHom)
S3method(pairs_min,data.frame)
S3method(pairs_min,default)
S3method(pairs_min,diagram)
S3method(pairs_min,matrix)
S3method(pairs_min,persistence)
S3method(pers_max,PHom)
S3method(pers_max,data.frame)
S3method(pers_max,default)
S3method(pers_max,diagram)
S3method(pers_max,matrix)
S3method(pers_max,persistence)
S3method(pers_min,PHom)
S3method(pers_min,data.frame)
S3method(pers_min,default)
S3method(pers_min,diagram)
S3method(pers_min,matrix)
S3method(pers_min,persistence)
S3method(pers_range,PHom)
S3method(pers_range,data.frame)
S3method(pers_range,default)
S3method(pers_range,diagram)
S3method(pers_range,matrix)
S3method(pers_range,persistence)
S3method(ph_dim,array)
S3method(ph_dim,data.frame)
S3method(ph_dim,default)
S3method(ph_dim,dist)
S3method(ph_dim,matrix)
S3method(ph_dim,ts)
S3method(prep,step_blur)
S3method(prep,step_pd_degree)
S3method(prep,step_pd_point_cloud)
S3method(prep,step_pd_raster)
S3method(prep,step_vpd_algebraic_functions)
S3method(prep,step_vpd_betti_curve)
S3method(prep,step_vpd_complex_polynomial)
S3method(prep,step_vpd_descriptive_statistics)
S3method(prep,step_vpd_euler_characteristic_curve)
S3method(prep,step_vpd_normalized_life_curve)
S3method(prep,step_vpd_persistence_block)
S3method(prep,step_vpd_persistence_image)
S3method(prep,step_vpd_persistence_landscape)
S3method(prep,step_vpd_persistence_silhouette)
S3method(prep,step_vpd_persistent_entropy_summary)
S3method(prep,step_vpd_tent_template_functions)
S3method(prep,step_vpd_tropical_coordinates)
S3method(print,step_blur)
S3method(print,step_pd_degree)
S3method(print,step_pd_point_cloud)
S3method(print,step_pd_raster)
S3method(print,step_vpd_algebraic_functions)
S3method(print,step_vpd_betti_curve)
S3method(print,step_vpd_complex_polynomial)
S3method(print,step_vpd_descriptive_statistics)
S3method(print,step_vpd_euler_characteristic_curve)
S3method(print,step_vpd_normalized_life_curve)
S3method(print,step_vpd_persistence_block)
S3method(print,step_vpd_persistence_image)
S3method(print,step_vpd_persistence_landscape)
S3method(print,step_vpd_persistence_silhouette)
S3method(print,step_vpd_persistent_entropy_summary)
S3method(print,step_vpd_tent_template_functions)
S3method(print,step_vpd_tropical_coordinates)
S3method(required_pkgs,step_blur)
S3method(required_pkgs,step_pd_degree)
S3method(required_pkgs,step_pd_point_cloud)
S3method(required_pkgs,step_pd_raster)
S3method(required_pkgs,step_tdarec)
S3method(required_pkgs,step_vpd_algebraic_functions)
S3method(required_pkgs,step_vpd_betti_curve)
S3method(required_pkgs,step_vpd_complex_polynomial)
S3method(required_pkgs,step_vpd_descriptive_statistics)
S3method(required_pkgs,step_vpd_euler_characteristic_curve)
S3method(required_pkgs,step_vpd_normalized_life_curve)
S3method(required_pkgs,step_vpd_persistence_block)
S3method(required_pkgs,step_vpd_persistence_image)
S3method(required_pkgs,step_vpd_persistence_landscape)
S3method(required_pkgs,step_vpd_persistence_silhouette)
S3method(required_pkgs,step_vpd_persistent_entropy_summary)
S3method(required_pkgs,step_vpd_tent_template_functions)
S3method(required_pkgs,step_vpd_tropical_coordinates)
S3method(tidy,step_blur)
S3method(tidy,step_pd_degree)
S3method(tidy,step_pd_point_cloud)
S3method(tidy,step_pd_raster)
S3method(tidy,step_vpd_algebraic_functions)
S3method(tidy,step_vpd_betti_curve)
S3method(tidy,step_vpd_complex_polynomial)
S3method(tidy,step_vpd_descriptive_statistics)
S3method(tidy,step_vpd_euler_characteristic_curve)
S3method(tidy,step_vpd_normalized_life_curve)
S3method(tidy,step_vpd_persistence_block)
S3method(tidy,step_vpd_persistence_image)
S3method(tidy,step_vpd_persistence_landscape)
S3method(tidy,step_vpd_persistence_silhouette)
S3method(tidy,step_vpd_persistent_entropy_summary)
S3method(tidy,step_vpd_tent_template_functions)
S3method(tidy,step_vpd_tropical_coordinates)
S3method(tunable,step_blur)
S3method(tunable,step_pd_point_cloud)
S3method(tunable,step_pd_raster)
S3method(tunable,step_vpd_algebraic_functions)
S3method(tunable,step_vpd_betti_curve)
S3method(tunable,step_vpd_complex_polynomial)
S3method(tunable,step_vpd_descriptive_statistics)
S3method(tunable,step_vpd_euler_characteristic_curve)
S3method(tunable,step_vpd_normalized_life_curve)
S3method(tunable,step_vpd_persistence_block)
S3method(tunable,step_vpd_persistence_image)
S3method(tunable,step_vpd_persistence_landscape)
S3method(tunable,step_vpd_persistence_silhouette)
S3method(tunable,step_vpd_persistent_entropy_summary)
S3method(tunable,step_vpd_tent_template_functions)
S3method(tunable,step_vpd_tropical_coordinates)
export(bandwidth)
export(birth_range)
export(blur)
export(blur_sigmas)
export(get_blur_range)
export(get_hom_range)
export(get_pairs_max)
export(get_pers_max_frac)
export(get_pers_min_mult)
export(hom_degree)
export(img_sigma)
export(life_support)
export(max_hom_degree)
export(num_bars)
export(num_bins)
export(num_coef)
export(num_levels)
export(pairs_max)
export(pairs_min)
export(pers_max)
export(pers_min)
export(pers_range)
export(ph_dim)
export(poly_type)
export(step_blur)
export(step_pd_degree)
export(step_pd_point_cloud)
export(step_pd_raster)
export(step_vpd_algebraic_functions)
export(step_vpd_betti_curve)
export(step_vpd_complex_polynomial)
export(step_vpd_descriptive_statistics)
export(step_vpd_euler_characteristic_curve)
export(step_vpd_normalized_life_curve)
export(step_vpd_persistence_block)
export(step_vpd_persistence_image)
export(step_vpd_persistence_landscape)
export(step_vpd_persistence_silhouette)
export(step_vpd_persistent_entropy_summary)
export(step_vpd_tent_template_functions)
export(step_vpd_tropical_coordinates)
export(tent_shift)
export(weight_func_pl)
export(weight_power)
import(recipes)
importFrom(dials,new_qual_param)
importFrom(dials,new_quant_param)
importFrom(dials,unknown)
importFrom(magrittr,"%>%")
importFrom(rlang,"%||%")
importFrom(rlang,on_load)
importFrom(rlang,run_on_load)
importFrom(scales,transform_log10)
importFrom(scales,transform_log1p)
