#' @title `tunable()` methods for tdavec package
#'
#' @description These functions define what parameters _can_ be tuned for
#'   specific steps. They also define the recommended objects from
#'   [dials][dials::dials-package] that can be used to generate new parameter
#'   values and other characteristics.
#'
#' @param x A recipe step object
#' @param ... Not used.
#' @name tunable_tdavec
#' @returns A tibble (class `tbl_df`).
#' @keywords internal
NULL
